﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Globalization;

namespace WpfApplication1
{
    class Person : INotifyPropertyChanged,IDataErrorInfo
    {
         [NonSerialized]
        private readonly DataErrorInfoSupport dataErrorInfoSupport;

        private string _Name;
        [Required(ErrorMessage="Please Enter Name")]
        public string Name
        {
            get { return _Name; }
            set
            {
                _Name = value;
                RaisePropertyChanged("Name");
            }
        }

        private DateTime? _DOB;
        public DateTime? DOB
        {
            get { return _DOB; }
            set
            {
                _DOB = value;
                RaisePropertyChanged("DOB");
            }
        }

        private int _Age;
        [Range(0,100,ErrorMessage="Please input valid range")]
        public int Age
        {
            get { return _Age; }
            set
            {
                _Age = value;
                RaisePropertyChanged("Age");
            }
        }

        public Person(string name,DateTime dob,int age)
        {
            dataErrorInfoSupport = new DataErrorInfoSupport(this);
            Name = name;
            DOB = dob;
            Age = age;
        }

        public Person()
        {
            dataErrorInfoSupport = new DataErrorInfoSupport(this);
        }

         #region IDataErrorInfo Members


        string IDataErrorInfo.Error { get { return dataErrorInfoSupport.Error; } }

        string IDataErrorInfo.this[string memberName] { get { return dataErrorInfoSupport[memberName]; } }


        #endregion

        #region INotifyPropertyChanged Members

            public event PropertyChangedEventHandler PropertyChanged;
            /// <summary>
            /// Raises the <see cref="E:PropertyChanged"/> event.
            /// </summary>
            /// <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/> instance containing the event data.</param>
            protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
            {
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, e);
                }
            }

            protected void RaisePropertyChanged(string propertyName)
            {

                OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
            }

        #endregion
    }
      public sealed class DataErrorInfoSupport : IDataErrorInfo
    {
        private readonly object instance;


        /// <summary>
        /// Initializes a new instance of the <see cref="DataErrorInfoSupport"/> class.
        /// </summary>
        /// <param name="instance">The instance.</param>
        /// <exception cref="ArgumentNullException">instance must not be <c>null</c>.</exception>
        public DataErrorInfoSupport(object instance)
        {
            if (instance == null) { throw new ArgumentNullException("instance"); }
            this.instance = instance;
        }


        /// <summary>
        /// Gets an error message indicating what is wrong with this object.
        /// </summary>
        /// <returns>An error message indicating what is wrong with this object. The default is an empty string ("").</returns>
        public string Error { get { return this[""]; } }

        /// <summary>
        /// Gets the error message for the property with the given name.
        /// </summary>
        /// <param name="memberName">The name of the property whose error message to get.</param>
        /// <returns>The error message for the property. The default is an empty string ("").</returns>
        public string this[string memberName]
        {
            get
            {
                List<ValidationResult> validationResults = new List<ValidationResult>();

                if (string.IsNullOrEmpty(memberName))
                {
                    Validator.TryValidateObject(instance, new ValidationContext(instance, null, null), validationResults, true);
                }
                else
                {
                    PropertyDescriptor property = TypeDescriptor.GetProperties(instance)[memberName];
                    if (property == null)
                    {
                        throw new ArgumentException(string.Format(CultureInfo.CurrentCulture,
                            "The specified member {0} was not found on the instance {1}", memberName, instance.GetType()));
                    }
                    Validator.TryValidateProperty(property.GetValue(instance),
                        new ValidationContext(instance, null, null) { MemberName = memberName }, validationResults);
                }

                StringBuilder errorBuilder = new StringBuilder();
                foreach (ValidationResult validationResult in validationResults)
                {
                    errorBuilder.AppendLine(validationResult.ErrorMessage);
                }

                return errorBuilder.ToString();
            }
        }
    }
}

